<?php

function _webform_defaults_wfso_field() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'pid' => 0,
    'weight' => 0,
    'value' => '',
    'mandatory' => 0,
    'extra' => array(
      'field_prefix' => '',
      'field_suffix' => '',
    ),
  );
}

/**
 * Implements _webform_theme_component().
 */
function _webform_theme_wfso_field() {
  return array(
    'webform_display_wfso_field' => array(
      'render element' => 'element',
      'file' => drupal_get_path('module', 'webform_select_opts') . '/components/wfso_field.inc',
    ),
  );
}

/**
 * Format the text output for this component.
 */
function theme_webform_display_wfso_field($variables) {
  
  dsm($variables);
  
  $element = $variables['element'];
  $component = $element['#webform_component'];

  // Flatten the list of options so we can get values easily. These options
  // may be translated by hook_webform_display_component_alter().
  $options = array();
  foreach ($element['#options'] as $key => $value) {
    if (is_array($value)) {
      foreach ($value as $subkey => $subvalue) {
        $options[$subkey] = $subvalue;
      }
    }
    else {
      $options[$key] = $value;
    }
  }

  $items = array();
  if ($component['extra']['multiple']) {
    foreach ((array) $element['#value'] as $option_value) {
      if ($option_value !== '') {
        // Administer provided values.
        if (isset($options[$option_value])) {
          $items[] = $options[$option_value];
        }
        // User-specified in the "other" field.
        else {
          $items[] = $option_value;
        }
      }
    }
  }
  else {
    if (isset($element['#value'][0]) && $element['#value'][0] !== '') {
      // Administer provided values.
      if (isset($options[$element['#value'][0]])) {
        $items[] = $options[$element['#value'][0]];
      }
      // User-specified in the "other" field.
      else {
        $items[] = $element['#value'][0];
      }
    }
  }

  if ($element['#format'] == 'html') {
    $output = count($items) > 1 ? theme('item_list', array('items' => $items)) : (isset($items[0]) ? $items[0] : ' ');
  }
  else {
    if (count($items) > 1) {
      foreach ($items as $key => $item) {
        $items[$key] = ' - ' . $item;
      }
      $output = implode("\n", $items);
    }
    else {
      $output = isset($items[0]) ? $items[0] : ' ';
    }
  }

  return $output;
}

function _webform_render_wfso_field($component, $value = NULL, $filter = FALSE) {
  global $user;
  $options = _wfso_options($component['nid'], $component['cid']);
  
  $node = isset($component['nid']) ? node_load($component['nid']) : NULL;

  $element = array(
    '#type' => 'select',
    '#title' => $filter ? _webform_filter_xss($component['name']) : $component['name'],
    '#options' => $options,
    '#default_value' => 0,
    '#required' => $component['mandatory'],
    '#weight' => $component['weight'],
    '#attributes' => @$component['extra']['attributes'],
    '#translatable' => array('title', 'description'),
  );

  // Add an e-mail class for identifying the difference from normal textfields.
  @$element['#attributes']['class'][] = 'wfso_field';

  // Enforce uniqueness.
  if (@$component['extra']['unique']) {
    $element['#element_validate'][] = 'webform_validate_unique';
  }

  if (isset($value)) {
    $element['#default_value'] = $value[0];
  }

  if (@$component['extra']['disabled']) {
    if ($filter) {
      $element['#attributes']['readonly'] = 'readonly';
    }
    else {
      $element['#disabled'] = TRUE;
    }
  }

  // Change the 'width' option to the correct 'size' option.
  if (@$component['extra']['width'] > 0) {
    $element['#size'] = $component['extra']['width'];
  }

  return $element;
}

/**
 * Implements _webform_display_component().
 */
function _webform_display_wfso_field($component, $value, $format = 'html') {
  return array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#theme' => 'webform_display_wfso_field',
    //'#theme_wrappers' => $format == 'html' ? array('webform_element') : array('webform_element_text'),
    '#format' => $format,
    '#value' => isset($value[0]) ? $value[0] : '',
    '#translatable' => array('title'),
  );
}

/*get options*/
function _wfso_options($nid, $cid) {
  $n_type = db_select('webform_select_opts_component', 's')
    -> fields('s', array('data'))
    -> condition('nid', $nid, '=')
    -> condition('cid', $cid, '=')
    -> execute() -> fetchField();
  
  if ($n_type) {
    $var = variable_get('webform_select_options_fields_' . $n_type, '');
    $field_val = unserialize($var);
    
    if ($var) {
      if ($field_val[0] == 'node_title') {
        $query = db_select('node', 'n');
        $query -> fields('n', array('nid', 'title'));
        $query -> condition('n.type', $n_type, '=');
        $query -> orderBy('title');
        $records = $query -> execute();
        
        foreach ($records AS $record) {
          $options[$record -> title] = $record -> title;
        }
      }
      else {
        $query = db_select('node', 'n');
        $query -> join('field_data_' . $field_val[0], 'm', 'n.nid = m.entity_id');
        $query -> fields('n', array('nid', 'title'));
        $query -> fields('m', array($field_val[0] . '_value'));
        $query -> condition('n.type', $n_type, '=');
        $query -> orderBy('title');
        $records = $query -> execute();
        
        $options = array();
        foreach ($records AS $record) {
          $record_arr = (array)$record;
          
          if ($record_arr[$field_val[0] . '_value'] != FALSE) {
            //if field VALUE is email
            if ($field_val[2]) {
              $options[$record_arr['nid']] = $record_arr['title'];
            }
            else {
              $options[$record_arr[$field_val[0] . '_value']] = $record_arr['title'];
            }
          }
        }
      }
    }
  }
  
  return $options;
}
